Module: BSV::Wallet::Interface

Defined in:
lib/bsv/wallet/interface.rb,
lib/bsv/wallet/interface/store.rb,
lib/bsv/wallet/interface/brc100.rb,
lib/bsv/wallet/interface/utxo_pool.rb,
lib/bsv/wallet/interface/proof_store.rb,
lib/bsv/wallet/interface/broadcast_queue.rb

Overview

BRC-100 abstract wallet interface — all 28 methods.

Include this module and override the methods your implementation supports. Unimplemented methods raise UnsupportedActionError.

The 28 methods are grouped into six functional areas matching the BRC-100 Interface Structure specification.

Examples:

class MyWallet
  include BSV::Wallet::Interface::BRC100

  def get_height(args = {}, originator: nil)
    { height: 800_000 }
  end
end

Defined Under Namespace

Modules: BRC100, BroadcastQueue, ProofStore, Store, UTXOPool