Class: BSV::Wallet::NullChainProvider

Inherits:
Object
  • Object
show all
Includes:
ChainProvider
Defined in:
lib/bsv/wallet_interface/null_chain_provider.rb

Overview

Default chain provider that raises for all blockchain queries.

Used when a WalletClient is constructed without a chain provider, allowing the wallet to function for transaction and crypto operations without requiring a blockchain connection.

Instance Method Summary collapse

Instance Method Details

#get_header(_height) ⇒ Object



17
18
19
# File 'lib/bsv/wallet_interface/null_chain_provider.rb', line 17

def get_header(_height)
  raise UnsupportedActionError, 'get_header (no chain provider configured)'
end

#get_heightObject



13
14
15
# File 'lib/bsv/wallet_interface/null_chain_provider.rb', line 13

def get_height
  raise UnsupportedActionError, 'get_height (no chain provider configured)'
end

#get_transaction(_txid) ⇒ Object



25
26
27
# File 'lib/bsv/wallet_interface/null_chain_provider.rb', line 25

def get_transaction(_txid)
  raise UnsupportedActionError, 'get_transaction (no chain provider configured)'
end

#get_utxos(_address) ⇒ Object



21
22
23
# File 'lib/bsv/wallet_interface/null_chain_provider.rb', line 21

def get_utxos(_address)
  raise UnsupportedActionError, 'get_utxos (no chain provider configured)'
end